/*
Copyright (C) 1996-1997 Id Software, Inc.

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  

See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/
// in_nitro.c -- input handler for the Nintendo DS using devkitARM
// (based on in_null.c)

#include <nds.h>

#define BYTE_DEFINED 1

typedef int qboolean; // FIX: For Nintendo DS using devkitARM - Oddly enough, this program requires that all its enums (like qboolean is supposed to be) measure sizeof(int) ( == 4) for it to work correctly.

#define QBOOLEAN_DEFINED 1

#include "quakedef.h"

cvar_t m_filter = {"m_filter","0"};

int window_center_x;

int window_center_y;

int previous_touch_x;

int previous_touch_y;

int old_mouse_x;

int old_mouse_y;

int mouse_x;

int mouse_y;

void IN_Init (void)
{
	window_center_x = 128;
	window_center_y = 96;
	previous_touch_x = window_center_x;
	previous_touch_y = window_center_y;
	old_mouse_x = window_center_x;
	old_mouse_y = window_center_y;
	Cvar_RegisterVariable(&m_filter);
}

void IN_Shutdown (void)
{
}

void IN_Commands (void)
{
}

void IN_Move (usercmd_t *cmd)
{
	uint32 k;
	touchPosition t;
	int x;
	int y;
	int mx;
	int my;

	scanKeys();
	k = keysHeld();

	if(key_dest != key_console)
	{
		// Use the touchscreen as a mouse substitute, but only if the console is not active: 

		if((k & KEY_TOUCH) == KEY_TOUCH)
		{
			touchRead(&t);
			x = t.px;
			y = t.py;
		} else
		{
			x = previous_touch_x;
			y = previous_touch_y;
		};
		
		mx = x - window_center_x;
		my = y - window_center_y;
		
		if (m_filter.value)
		{
			mouse_x = (mx + old_mouse_x) * 0.5;
			mouse_y = (my + old_mouse_y) * 0.5;
		}
		else
		{
			mouse_x = mx;
			mouse_y = my;
		};
		
		old_mouse_x = mx;
		old_mouse_y = my;
		
		mouse_x *= sensitivity.value;
		mouse_y *= sensitivity.value;

		if ( (in_strafe.state & 1) || (lookstrafe.value && (in_mlook.state & 1) ))
			cmd->sidemove += m_side.value * mouse_x;
		else
			cl.viewangles[YAW] -= m_yaw.value * mouse_x;

		if (in_mlook.state & 1)
			V_StopPitchDrift ();
			
		if ( (in_mlook.state & 1) && !(in_strafe.state & 1))
		{
			cl.viewangles[PITCH] += m_pitch.value * mouse_y;
			if (cl.viewangles[PITCH] > 80)
				cl.viewangles[PITCH] = 80;
			if (cl.viewangles[PITCH] < -70)
				cl.viewangles[PITCH] = -70;
		}
		else
		{
			if ((in_strafe.state & 1) && noclip_anglehack)
				cmd->upmove -= m_forward.value * mouse_y;
			else
				cmd->forwardmove -= m_forward.value * mouse_y;
		}

		previous_touch_x = x;
		previous_touch_y = y;

	};

}

