/*
Copyright (C) 1996-1997 Id Software, Inc.

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  

See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/
// sys_nitro.c -- system driver for the Nintendo DS using devkitARM
// (based on sys_null.c)

#include <nds.h>
#include <filesystem.h>
#include <fat.h>
#include <unistd.h>

#define BYTE_DEFINED 1

typedef int qboolean; // FIX: For Nintendo DS using devkitARM - Oddly enough, this program requires that all its enums (like qboolean is supposed to be) measure sizeof(int) ( == 4) for it to work correctly.

#define QBOOLEAN_DEFINED 1

#include "quakedef.h"
#include "errno.h"

qboolean isDedicated;

PrintConsole sys_console;

int sys_background_ID;

u16* sys_background_ptr;

keydest_t sys_previous_key_dest;

Keyboard* sys_onscreen_keyboard;

uint32 sys_previous_keys;

uint32 sys_current_weapon;

volatile int sys_elapsed_ticks;

volatile float sys_time_mark;

volatile float sys_elapsed_time;

extern qboolean	scr_drawdialog;

qboolean sys_previous_m_state_is_quit;

extern qboolean m_state_is_quit;

/*
===============================================================================

FILE IO

===============================================================================
*/

#define MAX_HANDLES             10
FILE    *sys_handles[MAX_HANDLES];

int             findhandle (void)
{
	int             i;
	
	for (i=1 ; i<MAX_HANDLES ; i++)
		if (!sys_handles[i])
			return i;
	Sys_Error ("out of handles");
	return -1;
}

/*
================
filelength
================
*/
int filelength (FILE *f)
{
	int             pos;
	int             end;

	pos = ftell (f);
	fseek (f, 0, SEEK_END);
	end = ftell (f);
	fseek (f, pos, SEEK_SET);

	return end;
}

int Sys_FileOpenRead (char *path, int *hndl)
{
	FILE    *f;
	int             i;
	
	i = findhandle ();

	f = fopen(path, "rb");
	if (!f)
	{
		*hndl = -1;
		return -1;
	}
	sys_handles[i] = f;
	*hndl = i;
	
	return filelength(f);
}

int Sys_FileOpenWrite (char *path)
{
	FILE    *f;
	int             i;
	
	i = findhandle ();

	f = fopen(path, "wb");
	if (!f)
		Sys_Error ("Error opening %s: %s", path,strerror(errno));
	sys_handles[i] = f;
	
	return i;
}

void Sys_FileClose (int handle)
{
	fclose (sys_handles[handle]);
	sys_handles[handle] = NULL;
}

void Sys_FileSeek (int handle, int position)
{
	fseek (sys_handles[handle], position, SEEK_SET);
}

void Sys_FileSeekCur (int handle, int offset)
{
	fseek (sys_handles[handle], offset, SEEK_CUR);
}

int Sys_FileRead (int handle, void *dest, int count)
{
	return fread (dest, 1, count, sys_handles[handle]);
}

int Sys_FileWrite (int handle, void *data, int count)
{
	return fwrite (data, 1, count, sys_handles[handle]);
}

int     Sys_FileTime (char *path)
{
	FILE    *f;
	
	f = fopen(path, "rb");
	if (f)
	{
		fclose(f);
		return 1;
	}
	
	return -1;
}

void Sys_mkdir (char *path)
{
}


/*
===============================================================================

SYSTEM IO

===============================================================================
*/

void Sys_MakeCodeWriteable (unsigned long startaddr, unsigned long length)
{
}


void Sys_Error (char *error, ...)
{
	va_list         argptr;

	printf ("Sys_Error: ");   
	va_start (argptr,error);
	vprintf (error,argptr);
	va_end (argptr);
	printf ("\n");

	exit (1);
}

void Sys_Printf (char *fmt, ...)
{
	va_list         argptr;
	
	va_start (argptr,fmt);
	vprintf (fmt,argptr);
	va_end (argptr);
}

void Sys_Quit (void)
{
	exit (0);
}

double Sys_FloatTime (void)
{
	static double t;
	
	t += 0.1;
	
	return t;
}

char *Sys_ConsoleInput (void)
{
	return NULL;
}

void Sys_Sleep (void)
{
}

void Sys_GenerateKey(int k)
{
	if(k > 0)
	{
		if(k == 10)
		{
			k = 13;
		} else if((k == '+')&&(!(sys_onscreen_keyboard->shifted)))
		{
			k = '=';
		};
		Key_Event(k, true);
		Key_Event(k, false);
	};
}

void Sys_SendKeyEvents (void)
{
	uint32 k;
	int kbk;

	scanKeys();
	k = keysHeld();
	if((sys_previous_keys & KEY_A) != (k & KEY_A))
	{
		Key_Event(K_ENTER, ((k & KEY_A) == KEY_A));
	};
	if((sys_previous_keys & KEY_B) != (k & KEY_B))
	{
		Key_Event(K_ESCAPE, ((k & KEY_B) == KEY_B));
	};
	if((sys_previous_keys & KEY_X) != (k & KEY_X))
	{
		if(((k & KEY_X) == KEY_X)&&(sys_current_weapon < 8))
		{
			sys_current_weapon++;
		};
		Key_Event('0' + sys_current_weapon, ((k & KEY_X) == KEY_X));
	};
	if((sys_previous_keys & KEY_Y) != (k & KEY_Y))
	{
		if(((k & KEY_Y) == KEY_Y)&&(sys_current_weapon > 1))
		{
			sys_current_weapon--;
		};
		Key_Event('0' + sys_current_weapon, ((k & KEY_Y) == KEY_Y));
	};
	if((sys_previous_keys & KEY_L) != (k & KEY_L))
	{
		Key_Event(K_MOUSE1, ((k & KEY_L) == KEY_L));
	};
	if((sys_previous_keys & KEY_R) != (k & KEY_R))
	{
		Key_Event(' ', ((k & KEY_R) == KEY_R));
	};
	if((sys_previous_keys & KEY_UP) != (k & KEY_UP))
	{
		Key_Event(K_UPARROW, ((k & KEY_UP) == KEY_UP));
	};
	if((sys_previous_keys & KEY_DOWN) != (k & KEY_DOWN))
	{
		Key_Event(K_DOWNARROW, ((k & KEY_DOWN) == KEY_DOWN));
	};
	if((sys_previous_keys & KEY_LEFT) != (k & KEY_LEFT))
	{
		Key_Event(K_LEFTARROW, ((k & KEY_LEFT) == KEY_LEFT));
	};
	if((sys_previous_keys & KEY_RIGHT) != (k & KEY_RIGHT))
	{
		Key_Event(K_RIGHTARROW, ((k & KEY_RIGHT) == KEY_RIGHT));
	};
	if((sys_previous_keys & KEY_SELECT) != (k & KEY_SELECT))
	{
		Key_Event('/', ((k & KEY_SELECT) == KEY_SELECT));
	};
	if((sys_previous_keys & KEY_START) != (k & KEY_START))
	{
		Key_Event(K_PAUSE, ((k & KEY_START) == KEY_START));
	};

	if((sys_previous_key_dest != key_dest)||(sys_previous_m_state_is_quit != m_state_is_quit))
	{
		if((key_dest == key_console)||(m_state_is_quit))
		{
			keyboardShow();
		} else
		{
			keyboardHide();
		};
		sys_previous_m_state_is_quit = m_state_is_quit;
		sys_previous_key_dest = key_dest;
	};

	sys_previous_keys = k;

	if(scr_drawdialog)
	{
		keyboardShow();
		kbk = keyboardGetChar();
		Sys_GenerateKey(kbk);
		keyboardHide();
	} else if((key_dest == key_console)||(m_state_is_quit))
	{
		kbk = keyboardUpdate();
		Sys_GenerateKey(kbk);
	};
}

void Sys_HighFPPrecision (void)
{
}

void Sys_LowFPPrecision (void)
{
}

// >>> FIX: For Nintendo DS using devkitARM
// New function to help local -> heap allocation issues
void* Sys_Malloc(int size, char* purpose)
{
	void* m;
	
	m = malloc(size);
	if(m == 0)
	{
		Sys_Error ("Sys_Malloc: %s - failed on %i bytes", purpose, size);
	};
	return m;
}
// <<< FIX

//=============================================================================

void Sys_VBlankHandler()
{
	sys_elapsed_ticks++;
	sys_elapsed_time = sys_elapsed_time + 16715.11263478648950875955477626;
}

void Sys_ExceptionHandler()
{
	Sys_Error("Hardware exception detected.");
}

int main (int argc, char* argv[])
{
	isDedicated = false;

	sys_previous_keys = 0;
	sys_current_weapon = 1;

	sys_elapsed_ticks = 0;
	sys_elapsed_time = 0;

	sys_previous_m_state_is_quit = 0;

	irqSet(IRQ_VBLANK, &Sys_VBlankHandler);

	setExceptionHandler(&Sys_ExceptionHandler);

	videoSetMode(MODE_3_2D);
	videoSetModeSub(MODE_0_2D);
	vramSetBankA(VRAM_A_MAIN_BG);
	vramSetBankB(VRAM_B_LCD);
	vramSetBankC(VRAM_C_SUB_BG);
	vramSetBankD(VRAM_D_LCD);
	vramSetBankE(VRAM_E_LCD);
	vramSetBankF(VRAM_F_LCD);
	vramSetBankG(VRAM_G_LCD);
	vramSetBankH(VRAM_H_LCD);
	vramSetBankI(VRAM_I_LCD);

	sys_console = *consoleInit(0, 1, BgType_Text4bpp, BgSize_T_256x256, 22, 3, false, true);

	sys_previous_key_dest = key_game;
	sys_onscreen_keyboard = keyboardInit(0, 0, BgType_Text4bpp, BgSize_T_256x512, 20, 0, false, true);
	keyboardHide();

	sys_background_ID = bgInit(3, BgType_Bmp8, BgSize_B8_256x256, 0, 0);
	sys_background_ptr = (u16*)bgGetGfxPtr(sys_background_ID);

	//if(!nitroFSInit())
	if(!fatInitDefault())
	{
		Sys_Error("Filesystem not enabled");
	};

	static quakeparms_t    parms;

	parms.memsize = 3005000;
	parms.membase = malloc (parms.memsize);
	parms.basedir = "fat:/";

	COM_InitArgv (argc, argv);

	parms.argc = com_argc;
	parms.argv = com_argv;

	printf ("Host_Init\n");
	Host_Init (&parms);
	while (1)
	{
		sys_time_mark = sys_elapsed_time;
		Host_Frame (0.1);
		do
		{
			swiWaitForVBlank();
		} while((sys_elapsed_time - sys_time_mark) < 100000.0);
		if(sys_elapsed_time > 10000000.0)
		{
			sys_elapsed_time = sys_elapsed_time - 10000000.0;
		};
	}
}


