/*
Copyright (C) 1996-1997 Id Software, Inc.

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  

See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/
// vid_nitro.c -- video driver for the Nintendo DS using devkitARM
// (based on vid_null.c)

#include <nds.h>

#define BYTE_DEFINED 1

typedef int qboolean; // FIX: For Nintendo DS using devkitARM - Oddly enough, this program requires that all its enums (like qboolean is supposed to be) measure sizeof(int) ( == 4) for it to work correctly.

#define QBOOLEAN_DEFINED 1

#include "quakedef.h"
#include "d_local.h"

viddef_t	vid;				// global video state

#define	BASEWIDTH	320
#define	BASEHEIGHT	200

#define	VIDEOWIDTH	256
#define	VIDEOHEIGHT	192

#define OUTERFRAMEWIDTH  ((BASEWIDTH - VIDEOWIDTH) / 2)
#define OUTERFRAMEHEIGHT ((BASEHEIGHT - VIDEOHEIGHT) / 2)

byte	vid_buffer[BASEWIDTH*BASEHEIGHT];

//short	zbuffer[BASEWIDTH*BASEHEIGHT];
short* zbuffer = (short*)VRAM_B;

//byte	surfcache[256*1024];
byte	surfcache[60*1024];

//unsigned short	d_8to16table[256];
unsigned short*	d_8to16table = ((unsigned short*)VRAM_B) + (BASEWIDTH*BASEHEIGHT);

//unsigned	d_8to24table[256];
unsigned*	d_8to24table = ((unsigned*)VRAM_B) + ((BASEWIDTH*BASEHEIGHT) / 2) + 128;

extern u16* sys_background_ptr;

void	VID_SetPalette (unsigned char *palette)
{
	int i;
	int j;
	char r;
	char g;
	char b;

	j = 0;
	for(i = 0; i < 256; i++)
	{
		r = palette[j];
		j++;
		g = palette[j];
		j++;
		b = palette[j];
		j++;
		d_8to16table[i] = 0x8000 | (((b >> 3) & 0x1F) << 10) | (((g >> 3) & 0x1F) << 5) | ((r >> 3) & 0x1F);
		d_8to24table[i] = 0xFF000000 | (b << 16) | (g << 8) | r;
	};
	dmaCopy(d_8to16table, BG_PALETTE, 512);
}

void	VID_ShiftPalette (unsigned char *palette)
{
	VID_SetPalette(palette);
}

void	VID_Init (unsigned char *palette)
{
	vid.maxwarpwidth = vid.width = vid.conwidth = BASEWIDTH;
	vid.maxwarpheight = vid.height = vid.conheight = BASEHEIGHT;
	vid.aspect = 1.0;
	vid.numpages = 1;
	vid.colormap = host_colormap;
	vid.fullbright = 256 - LittleLong (*((int *)vid.colormap + 2048));
	vid.buffer = vid.conbuffer = vid_buffer;
	vid.rowbytes = vid.conrowbytes = BASEWIDTH;
	
	VID_SetPalette(palette);

	d_pzbuffer = zbuffer;
	D_InitCaches (surfcache, sizeof(surfcache));
}

void	VID_Shutdown (void)
{
}

void	VID_Update (vrect_t *rects)
{
	vrect_t* r;
	int x;
	int y;
	int dx;
	int dy;
	int w;
	int h;
	int bx;
	int vx;
	int by;
	int vy;
	byte* p;
	byte* q;
	int i;

	r = rects;
	while(r != 0)
	{
		x = r->x;
		y = r->y;
		if(x >= OUTERFRAMEWIDTH)
		{
			dx = 0;
		} else
		{
			dx = OUTERFRAMEWIDTH - x;
		};
		if(y >= OUTERFRAMEHEIGHT)
		{
			dy = 0;
		} else
		{
			dy = OUTERFRAMEHEIGHT - y;
		};
		w = r->width - dx;
		if(w > VIDEOWIDTH)
		{
			w = VIDEOWIDTH;
		};
		h = r->height - dy;
		if(h > VIDEOHEIGHT)
		{
			h = VIDEOHEIGHT;
		};
		bx = x + dx;
		vx = bx - OUTERFRAMEWIDTH;
		by = y + dy;
		vy = by - OUTERFRAMEHEIGHT;
		p = vid_buffer + (by * BASEWIDTH) + bx;
		q = ((byte*)(sys_background_ptr)) + (vy * VIDEOWIDTH) + vx;
		for(i = 0; i < h; i++)
		{
			dmaCopy(p, q, w);
			p += BASEWIDTH;
			q += VIDEOWIDTH;
		};
		r = r->pnext;
	};
}

/*
================
D_BeginDirectRect
================
*/
void D_BeginDirectRect (int x, int y, byte *pbitmap, int width, int height)
{
	int dx;
	int dy;
	int w;
	int h;
	int bx;
	int vx;
	int by;
	int vy;
	byte* p;
	byte* q;
	int i;

	if(pbitmap != 0)
	{
		if(x >= OUTERFRAMEWIDTH)
		{
			dx = 0;
		} else
		{
			dx = OUTERFRAMEWIDTH - x;
		};
		if(y >= OUTERFRAMEHEIGHT)
		{
			dy = 0;
		} else
		{
			dy = OUTERFRAMEHEIGHT - y;
		};
		w = width - dx;
		if(w > VIDEOWIDTH)
		{
			w = VIDEOWIDTH;
		};
		h = height - dy;
		if(h > VIDEOHEIGHT)
		{
			h = VIDEOHEIGHT;
		};
		bx = dx;
		vx = x + dx - OUTERFRAMEWIDTH;
		by = dy;
		vy = y + dy - OUTERFRAMEHEIGHT;
		p = pbitmap + (by * width) + bx;
		q = ((byte*)(sys_background_ptr)) + (vy * VIDEOWIDTH) + vx;
		for(i = 0; i < h; i++)
		{
			dmaCopy(p, q, w);
			p += width;
			q += VIDEOWIDTH;
		};
	};
}


/*
================
D_EndDirectRect
================
*/
void D_EndDirectRect (int x, int y, int width, int height)
{
}


